#!/usr/bin/env python3
# coding: utf-8

from punchline_python.core.holders.input_holder import InputHolder
from punchline_python.core.holders.output_holder import OutputHolder
from punchline_python.core.node import AbstractNode


class Sqlite3OutputSettings(object):

    CREATE_TABLE_IF_NOT_EXIST: str = "create_table_if_not_exist"
    DATABASE_PATH: str = "database_path"
    TABLE_COLUMNS: str = "table_columns"
    TABLE_NAME: str = "table_name"


class Sqlite3Output(AbstractNode):

    @AbstractNode.declare_param(name=Sqlite3OutputSettings.CREATE_TABLE_IF_NOT_EXIST, required=False, default=True)
    @AbstractNode.declare_param(name=Sqlite3OutputSettings.DATABASE_PATH, required=True)
    @AbstractNode.declare_param(name=Sqlite3OutputSettings.TABLE_COLUMNS, required=True)
    @AbstractNode.declare_param(name=Sqlite3OutputSettings.TABLE_NAME, required=True)
    def __init__(self) -> None:
        super().__init__()

    @staticmethod
    def create_table_if_not_exists_sql(table_name: str, col_and_types: dict) -> str:
        columns: str = ""
        for key, value in col_and_types.items():
            columns = columns + str(key) + " " + str(value) + ","
        columns = columns[:-1]
        if len(columns) == 0:
            raise Exception("no columns specified")
        else:
            return "CREATE TABLE {} ({})".format(table_name, columns)

    @staticmethod
    def insert_query(table_name: str, num_of_columns: int):
        cols: str = ""
        for i in range(0, num_of_columns):
            cols = cols + "?,"
        cols = cols[:-1]
        return "INSERT INTO {} VALUES ({})".format(table_name, cols)

    def execute(self, input_data: InputHolder, output_data: OutputHolder) -> None:
        import sqlite3

        columns: dict = self.settings.get(Sqlite3OutputSettings.TABLE_COLUMNS)
        table_name: str = self.settings.get(Sqlite3OutputSettings.TABLE_NAME)
        db_path: str = self.settings.get(Sqlite3OutputSettings.DATABASE_PATH)
        create_table_not_exists: bool = self.settings.get(Sqlite3OutputSettings.CREATE_TABLE_IF_NOT_EXIST)
        try:
            conn: sqlite3.Connection = sqlite3.connect(db_path)
            sql: sqlite3.Cursor = conn.cursor()

            if create_table_not_exists:
                try:
                    create_query: str = Sqlite3Output.create_table_if_not_exists_sql(table_name, columns)
                    sql.execute(create_query)
                    conn.commit()
                except sqlite3.Error as e:
                    if "already exists" in str(e):
                        pass
                    else:
                        raise e

            for k, v in input_data.streams.items():

                if type(v) == list:
                    sql.executemany(Sqlite3Output.insert_query(table_name, len(columns)), v)
                    conn.commit()
                else:
                    print("invalid input type for stream " + k)

        except sqlite3.Error as e:
            raise e
        finally:
            conn.close()


